@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = 0.14;
+our $VERSION = 0.15;
 
 =head1 NAME
 
@@ -159,7 +159,7 @@ They have nothing to do with serving the request.
 use Carp;
 use Scalar::Util qw(blessed looks_like_number);
 use Encode;
-use POSIX qw(strftime);
+use URI::Escape;
 use parent qw(Exporter);
 
 our @EXPORT_OK = qw(neaf_err neaf get post);
@@ -167,7 +167,7 @@ our %EXPORT_TAGS = (
     sugar => [qw[neaf get post]],
 );
 
-use MVC::Neaf::Util qw(http_date canonize_path path_prefixes);
+use MVC::Neaf::Util qw(http_date canonize_path path_prefixes run_all run_all_nodie);
 use MVC::Neaf::Request;
 
 our $Inst = __PACKAGE__->new;
@@ -272,7 +272,7 @@ sub route {
     } else {
         $profile{has_path_info_regex} = 1;
     };
-    $profile{path_info_regex_regex}  = qr#^/*($args{path_info_regex})$#;
+    $profile{path_info_regex}  = qr#^/*($args{path_info_regex})$#;
 
     # Just for information
     $profile{path}        = $path;
@@ -413,13 +413,16 @@ Return value from callback is ignored.
 
 Dying in callback is treated the same way as in normal controller sub.
 
+B<DEPRECATED>. Use C<Neaf-E<gt>add_hook( pre_route =E<gt> ... )> instead.
+
 =cut
 
 sub pre_route {
     my ($self, $code) = @_;
     $self = $Inst unless ref $self;
 
-    $self->{pre_route} = $code;
+    carp ("DEPRECATED: pre_route(): use add_hook( pre_route => CODE ) instead");
+    $self->add_hook( pre_route => $code );
     return $self;
 };
 
@@ -745,6 +748,33 @@ sub run {
 
     $self->{route_re} ||= $self->_make_route_re;
 
+    # initialize stuff if first run
+    # TODO don't allow modification after lock
+    if (!$self->{lock}) {
+        if (my $engine = $self->{session_handler}) {
+            $self->add_hook( pre_route => sub {
+                $_[0]->_set_session_handler( $engine );
+            }, prepend => 1 );
+            if (my $key = $self->{session_view_as}) {
+                $self->add_hook( pre_render => sub {
+                    my $sess = $_[0]->session(1);
+                    $sess and $_[0]->reply->{$key} = $sess;
+                }, prepend => 1 );
+            };
+        };
+        if (my $engine = $self->{stat}) {
+            $self->add_hook( pre_route => sub {
+                $engine->record_start;
+            }, prepend => 1);
+            $self->add_hook( pre_content => sub {
+                $engine->record_controller( $_[0]->script_name );
+            }, prepend => 1);
+            $self->add_hook( pre_reply => sub {
+                $engine->record_finish($_[0]->reply->{-status}, $_[0]);
+            }, prepend => 1);
+        };
+    };
+
     if (defined wantarray) {
         # The run method is being called in non-void context
         # This is the case for PSGI, but not CGI (where it's just
@@ -865,7 +895,7 @@ C<neaf-E<gt>method_name> is the equivalent of C<MVC::Neaf-E<gt>method_name>.
 
 =head2 neaf shortcut => @options;
 
-Shorter alias to method described above. Currently supported:
+Shorter alias to methods described above. Currently supported:
 
 =over
 
@@ -875,24 +905,53 @@ Shorter alias to method described above. Currently supported:
 
 =item * view - C<load_view>
 
+=item * hook - C<add_hook>
+
+=item * session - C<set_session_handler>
+
+=item * default - C<set_path_defaults>
+
+=item * alias   - C<alias>
+
+=item * static  - C<static>
+
 =back
 
+Also, passing a 3-digit number will trigger C<set_error_handler>,
+and passing a hook phase (see below) will result in setting a hook.
+
 =cut
 
 my %method_shortcut = (
-    route => 'route',
-    error => 'set_error_handler',
-    view  => 'load_view',
+    route    => 'route',
+    error    => 'set_error_handler',
+    view     => 'load_view',
+    hook     => 'add_hook',
+    session  => 'set_session_handler',
+    default  => 'set_path_defaults',
+    alias    => 'alias',
+    static   => 'static',
 );
+my %hook_phases;
+$hook_phases{$_}++ for qw(pre_route pre_logic pre_content pre_render pre_reply pre_cleanup);
 
 sub neaf(@) { ## no critic # DSL
     return $MVC::Neaf::Inst unless @_;
 
     my ($action, @args) = @_;
 
+    if ($action =~ /^\d\d\d$/) {
+        unshift @args, $action;
+        $action = 'error';
+    };
+    if ($hook_phases{$action}) {
+        unshift @args, $action;
+        $action = 'hook';
+    };
+
     my $method = $method_shortcut{$action};
     croak "neaf: don't know how to handle '$action'"
-        unless $method;
+        unless $method and MVC::Neaf->can($method);
 
     return MVC::Neaf->$method( @args );
 };
@@ -951,12 +1010,28 @@ Multiple locations may be supplied via C<[ /foo, /bar ...]>
 =item * method => 'METHOD' || [ list ]
 List of request HTTP methods to which given hook applies.
 
+=item * prepend => 0|1 - if true, move hook execution to earlier stage.
+Note that this does NOT override path precedence.
+
 =back
 
 =head2 HOOK PHASES
 
 This list of phases MAY change in the future.
 
+=head3 pre_route
+
+Executed AFTER the event has been received, but BEFORE the path has been
+resolved and handler found.
+
+Dying in this phase stops both further hook processing and controller execution.
+Instead, the corresponding error handler is executed right away.
+
+Setting C<path> and C<exclude> is not available on this stage.
+
+May be useful for mangling path.
+Use C<$request-E<gt>set_full_path($new_path)> if you need to.
+
 =head3 pre_logic
 
 Executed AFTER finding the correct route, but BEFORE processing the main
@@ -986,6 +1061,14 @@ C<reply()> hash is available at this stage.
 
 Dying is ignored, only producing a warning.
 
+=head3 pre_render
+
+This hook is run BEFORE content rendering is performed, and ONLY IF
+the content is going to be rendered,
+i.e. no C<-content> key set in response hash on previous stages.
+
+Dying will stop rendering, resulting in a template error instead.
+
 =head3 pre_reply
 
 This hook is run AFTER the headers have been generated, but BEFORE the reply is
@@ -1012,9 +1095,7 @@ Dying is ignored, only producing a warning.
 =cut
 
 my %add_hook_args;
-$add_hook_args{$_}++ for qw(method path exclude);
-my %hook_phases;
-$hook_phases{$_}++ for qw(pre_logic pre_content pre_reply pre_cleanup);
+$add_hook_args{$_}++ for qw(method path exclude prepend);
 
 sub add_hook {
     my ($self, $phase, $code, %opt) = @_;
@@ -1027,9 +1108,21 @@ sub add_hook {
         unless $hook_phases{$phase};
 
     _listify( \$opt{method}, qw( GET HEAD POST PUT PATCH DELETE ) );
+    if ($phase eq 'pre_route') {
+        # handle pre_route separately
+        $self->_croak("cannot specify paths/excludes for $phase")
+            if defined $opt{path} || defined $opt{exclude};
+        foreach( @{ $opt{method} } ) {
+            my $where = $self->{pre_route}{$_} ||= [];
+            $opt{prepend} ? unshift @$where, $code : push @$where, $code;
+        };
+        return $self;
+    };
+
     _listify( \$opt{path}, '/' );
     _listify( \$opt{exclude} );
     @{ $opt{path} } = map { canonize_path($_) } @{ $opt{path} };
+    @{ $opt{exclude} } = map { canonize_path($_) } @{ $opt{exclude} };
 
     $opt{caller} = [ caller(0) ]; # where the hook was set
     $opt{phase}  = $phase; # just for information
@@ -1040,7 +1133,7 @@ sub add_hook {
     foreach my $method ( @{$opt{method}} ) {
         foreach my $path ( @{$opt{path}} ) {
             my $where = $self->{hooks}{$method}{$path}{$phase} ||= [];
-            push @$where, \%opt;
+            $opt{prepend} ? unshift @$where, \%opt : push @$where, \%opt;
         };
     };
 
@@ -1196,38 +1289,36 @@ sub handle_request {
     my ($self, $req) = @_;
     $self = $Inst unless ref $self;
 
-    exists $self->{stat} and $self->{stat}->record_start;
-
     # ROUTE REQUEST
     my $route;
     my $data = eval {
-        # Inject session handler into request object
-        $req->_set_session_handler( $self->{session_handler} )
-            if exists $self->{session_handler};
-
+        my $method = $req->method;
         # Try running the pre-routing callback.
-        if (exists $self->{pre_route}) {
-            $self->{pre_route}->( $req );
-        };
+        run_all( $self->{pre_route}{$method}, $req )
+            if (exists $self->{pre_route}{$method});
 
         # Lookup the rules for the given path
         $req->path =~ $self->{route_re} and my $node = $self->{route}{$1}
             or die "404\n";
-        unless ($route = $node->{ $req->method }) {
+        unless ($route = $node->{ $method }) {
             $req->set_header( Allow => join ", ", keys %$node );
             die "405\n";
         };
 
         # TODO optimize this or do smth. Still MUST keep route_re a prefix tree
-        my ($path, $path_info_regex) = ($1, $2);
-        $path_info_regex =~ $route->{path_info_regex_regex}
+        my ($path, $path_info) = ($1, $2);
+        if ($path_info =~ /%/) {
+            $path_info = decode_utf8( uri_unescape( $path_info ) );
+        };
+        $path_info =~ $route->{path_info_regex}
             or die "404\n";
-        $req->set_full_path( $path, $path_info_regex, !$route->{has_path_info_regex} );
+        $req->set_full_path( $path, $path_info, !$route->{has_path_info_regex} );
         $self->_post_setup( $route )
             unless exists $route->{lock};
 
         # execute hooks
-        $_->($req) for @{ $route->{hooks}{pre_logic} };
+        run_all( $route->{hooks}{pre_logic}, $req)
+            if exists $route->{hooks}{pre_logic};
         # Run the controller!
         return $route->{code}->($req);
     };
@@ -1236,25 +1327,22 @@ sub handle_request {
         # post-process data - fill in request(RD) & global(GD) defaults.
         # TODO fill in per-location defaults, too - but do we need them?
         my $RD = $req->get_default;
-        my $GD = $route->{default} ||= $self->_make_defaults( $route );
+        my $GD = $route->{default};
         exists $data->{$_} or $data->{$_} = $RD->{$_} for keys %$RD;
         exists $data->{$_} or $data->{$_} = $GD->{$_} for keys %$GD;
     } else {
         # Fall back to error page
+        # TODO $req->clear; - but don't kill cleanup hooks
         $data = $self->_error_to_reply( $req, $@, $route->{caller} );
     };
 
     # END ROUTE REQUEST
 
     $req->_set_reply( $data );
-    exists $self->{stat}
-        and $self->{stat}->record_controller($req->script_name);
     if (exists $route->{hooks}{pre_content}) {
-        foreach my $hook ( @{ $route->{hooks}{pre_content} } ) {
-            if (!eval { $hook->( $req ); 1 }) {
-                $self->_log_error( "pre_content hook", $@ );
-            };
-        };
+        run_all_nodie( $route->{hooks}{pre_content}, sub {
+                $self->_log_error( "pre_content hook", $@ )
+        }, $req );
     };
 
     # PROCESS REPLY
@@ -1264,22 +1352,20 @@ sub handle_request {
     my $content = \$data->{-content};
     if( !defined $$content) {
         my $view = $self->load_view( $data->{-view} || $route->{view} );
-        # TODO move session to hook
-        if ( my $key = $self->{session_view_as} and my $sess = $req->session(1) ) {
-            $data->{ $key } = $sess;
-        };
         eval {
+            run_all( $route->{hooks}{pre_render}, $req )
+                if exists $route->{hooks}{pre_render};
             ($$content, my $type) = $view->render( $data );
             $data->{-type} ||= $type;
         };
         if (!defined $$content) {
-            # TODO $req->clear in case of error
+            # TODO $req->clear; - but don't kill cleanup hooks
             $self->_log_error( view => $@ );
             $data = {
                 -status => 500,
                 -type   => "text/plain",
             };
-            $$content = "Template error.";
+            $$content = "Template error."; # TODO configurable
         };
     };
 
@@ -1317,18 +1403,14 @@ sub handle_request {
 
     # END PROCESS REPLY
 
-    if (exists $route->{hooks}{pre_reply}) {
-        foreach my $hook ( @{ $route->{hooks}{pre_reply} } ) {
-            if (!eval { $hook->( $req ); 1 }) {
-                $self->_log_error( "pre_reply hook", $@ );
-            };
-        };
-    };
-    exists $self->{stat}
-        and $self->{stat}->record_finish($data->{-status}, $req);
     if (exists $route->{hooks}{pre_cleanup}) {
         $req->postpone( $route->{hooks}{pre_cleanup} );
     };
+    if (exists $route->{hooks}{pre_reply}) {
+        run_all_nodie( $route->{hooks}{pre_reply}, sub {
+                $self->_log_error( "pre_reply hook", $@ )
+        }, $req );
+    };
 
     # DISPATCH CONTENT
 
